@@ -145,8 +145,9 @@ static void update_hosts(void) {
 		&reply, "isqqt", 0 /* any */, PACSERVE "." MDNS_DOMAIN,
 		DNS_CLASS_IN, DNS_TYPE_PTR, SD_RESOLVED_NO_SYNTHESIZE|SD_RESOLVED_NO_ZONE);
 	if (r < 0) {
-		if (verbose > 0)
-			write_log(stderr, "Failed to resolve record: %s\n", error.message);
+		if (errno != EAGAIN || verbose > 0)
+			write_log(stderr, "Failed to trigger caching record: %s (%s)\n",
+				error.message, strerror(errno));
 		sd_bus_error_free(&error);
 		goto finish;
 	}
@@ -200,7 +201,7 @@ static void update_hosts(void) {
 		}
 
 		if (!ignore)
-			update_hosts_on_interface(bus, intf->if_index);
+			update_hosts_on_interface(bus, intf->if_index, intf->if_name);
 
 		if (quit)
 			break;
@@ -227,7 +228,7 @@ fast_finish:
 }
 
 /*** update_hosts_on_interface ***/
-static void update_hosts_on_interface(sd_bus *bus, const unsigned int if_index) {
+static void update_hosts_on_interface(sd_bus *bus, const unsigned int if_index, const char *if_name) {
 	sd_bus_error error = SD_BUS_ERROR_NULL;
 	sd_bus_message *reply_record = NULL;
 	uint64_t flags;
@@ -238,8 +239,9 @@ static void update_hosts_on_interface(sd_bus *bus, const unsigned int if_index)
 		&reply_record, "isqqt", if_index, PACSERVE "." MDNS_DOMAIN,
 		DNS_CLASS_IN, DNS_TYPE_PTR, SD_RESOLVED_NO_SYNTHESIZE|SD_RESOLVED_NO_ZONE);
 	if (r < 0) {
-		if (verbose > 0)
-			write_log(stderr, "Failed to resolve record: %s\n", error.message);
+		if (errno != EAGAIN || verbose > 0)
+			write_log(stderr, "Failed to resolve record on %s: %s (%s)\n",
+				if_name, error.message, strerror(errno));
 		sd_bus_error_free(&error);
 		goto finish;
 	}
@@ -255,7 +257,7 @@ static void update_hosts_on_interface(sd_bus *bus, const unsigned int if_index)
 		size_t length;
 		char *peer;
 		const char *canonical, *discard;
-		uint8_t match = 0, ignore = 0;
+		uint8_t match = 0;
 
 		r = sd_bus_message_enter_container(reply_record, 'r', "iqqay");
 		if (r < 0)
@@ -279,10 +281,10 @@ static void update_hosts_on_interface(sd_bus *bus, const unsigned int if_index)
 		/* service START */
 		r = sd_bus_call_method(bus, "org.freedesktop.resolve1", "/org/freedesktop/resolve1",
 			"org.freedesktop.resolve1.Manager", "ResolveService", &error,
-			&reply_service, "isssit", 0 /* any */, "", "", peer, AF_UNSPEC, UINT64_C(0));
+			&reply_service, "isssit", if_index, "", "", peer, AF_UNSPEC, UINT64_C(0));
 		if (r < 0) {
-			if (verbose > 0)
-				write_log(stderr, "Failed to resolve service '%s': %s\n", peer, error.message);
+			write_log(stderr, "Failed to resolve service '%s' on %s: %s (%s)\n",
+				peer, if_name, error.message, strerror(errno));
 			sd_bus_error_free(&error);
 			goto finish_service;
 		}
@@ -382,12 +384,6 @@ static void update_hosts_on_interface(sd_bus *bus, const unsigned int if_index)
 		if (r < 0)
 			goto parse_failure_service;
 
-		if (ignore > 0) {
-			if (verbose > 0)
-				write_log(stdout, "Host %s is on an ignored interface.\n", canonical);
-			goto finish_service;
-		}
-
 		if (match < DNS_SRV_TXT_MATCH_ALL) {
 			if (verbose > 0)
 				write_log(stdout, "Host %s does not match distribution and/or architecture.\n", canonical);
@@ -461,25 +457,6 @@ update:
 	return EXIT_SUCCESS;
 }
 
-/*** remove_host ***/
-/* currently unused, but could become important if continuous
-   mDNS querying becomes available again on day...
-static int remove_host(const char * host) {
-	struct hosts * hosts_ptr = hosts;
-
-	while (hosts_ptr->host != NULL) {
-		if (strcmp(hosts_ptr->host, host) == 0) {
-			if (verbose > 0)
-				write_log(stdout, "Marking host %s offline\n", host);
-			hosts_ptr->online = 0;
-			break;
-		}
-		hosts_ptr = hosts_ptr->next;
-	}
-
-	return EXIT_SUCCESS;
-} */
-
 /*** get_http_code ***/
 static void * get_http_code(void * data) {
 	struct request * request = (struct request *)data;
@@ -933,6 +910,10 @@ static void sighup_callback(int signal) {
 static void sigusr_callback(int signal) {
 	struct ignore_interfaces * ignore_interfaces_ptr = ignore_interfaces;
 	struct hosts * hosts_ptr = hosts;
+	struct timeval tv;
+
+	/* initialize struct timeval */
+	gettimeofday(&tv, NULL);
 
 	write_log(stdout, "Received signal '%s', dumping state.\n", strsignal(signal));
 
@@ -953,10 +934,13 @@ static void sigusr_callback(int signal) {
 	if (hosts_ptr->host == NULL)
 		write_log(stdout, " (none)\n");
 	while (hosts_ptr->host != NULL) {
-		write_log(stdout, " -> %s (%s, %s, port: %d, finds: %d, bad: %d)\n",
-			hosts_ptr->host, hosts_ptr->mdns ? "mdns" : "static",
-			hosts_ptr->online ? "online" : "offline", hosts_ptr->port,
-			hosts_ptr->finds, hosts_ptr->badcount);
+		uint8_t not_avail = (hosts_ptr->mdns && !hosts_ptr->online) || (hosts_ptr->badcount &&
+			(hosts_ptr->badtime + hosts_ptr->badcount * BADTIME) > tv.tv_sec) ? 1 : 0;
+
+		write_log(stdout, " -> %s%s%s (%s, %s, port: %d, finds: %d, bad: %d)\n",
+			not_avail ? "[" : "", hosts_ptr->host, not_avail ? "]" : "",
+			hosts_ptr->mdns ? "mdns" : "static", hosts_ptr->online ? "online" : "offline",
+			hosts_ptr->port, hosts_ptr->finds, hosts_ptr->badcount);
 
 		hosts_ptr = hosts_ptr->next;
 	}
